#ifndef _GEOMETRIC_H_
#define _GEOMETRIC_H_

#include "rectangle.h"

namespace vsip {

#define USE_IP_RECT

/** replace nr_line in libvu-II */
struct line_2d {
    int             id;
    double          x1;
    double          y1;
    double          x2;
    double          y2;
    struct line_2d  *next;
};

struct point_2d {
    int x;
    int y;
};

struct fpoint_2d {
    double x;
    double y;
};

struct point_3d {
    double x;
    double y;
    double z;
};

struct vector_2d {
    int x;
    int y;
};

struct fvector_2d {
  double x;
  double y;
};

struct vector_3d {
    double x;
    double y;
    double z;
};

struct h_segment {
    int line;
    int left;
    int right;
    struct segment *next_seg;
};

struct intersection {
  double x1, y1;
  double x2, y2;
  double distance;
  int type; //1: One point intersection (Stored in (x1,y1) )
            //0: Segment intersection (Limits stored in {(x1,y1);(x2,y2)} )
};


struct segment_2d {
    struct point_2d first_point;
    struct point_2d last_point;
    struct segment_2d *next_seg;
};

struct segment_3d {
    struct point_3d first_point;
    struct point_3d last_point;
    struct segment_3d *next_seg;
};


struct polygon_2d {
    int n_points;
    struct rectangle bounding_box;
    int flag_max;
    struct polygon_2d *next;
    int n_points_max;
    struct point_2d *pts_array;
};

struct fpolygon_2d {
    int n_points;
    struct frectangle bounding_box;
    int flag_max;
    struct fpolygon_2d *next;
    int n_points_max;
    struct fpoint_2d *pts_array;
};

#define POLYGON_MEM_SIZE(n)	(sizeof(struct polygon_2d) + \
				 (n) * sizeof(struct point_2d))

// for 2d and 3d polygon...
#define POLYGON_N_POINTS(pl)		((pl)->n_points)
#define POLYGON_MAX_PTS(pl)		((pl)->n_points_max)
#define POLYGON_NTH_POINT(pl, n)	((pl)->pts_array + (n))
#define POLYGON_NEXT(pl)                ((pl)->next)

// only for 2d-polygon 
#define POLYGON_2D_BB(pl)		    ((pl)->bounding_box)
#define POLYGON_2D_BB_X(pl)		    RECT_X(&((pl)->bounding_box))
#define POLYGON_2D_BB_Y(pl)		    RECT_Y(&((pl)->bounding_box))
#define POLYGON_2D_BB_WIDTH(pl)		RECT_WIDTH(&((pl)->bounding_box))
#define POLYGON_2D_BB_HEIGHT(pl)	RECT_HEIGHT(&((pl)->bounding_box))
#define POLYGON_2D_BB_XLEFT(pl)		RECT_XLEFT(&((pl)->bounding_box))
#define POLYGON_2D_BB_XRIGHT(pl)	RECT_XRIGHT(&((pl)->bounding_box))
#define POLYGON_2D_BB_YBOTTOM(pl)	RECT_YBOTTOM(&((pl)->bounding_box))
#define POLYGON_2D_BB_YTOP(pl)	    RECT_YTOP(&((pl)->bounding_box))
#define POLYGON_2D_BB_XCENTER(pl)	(POLYGON_2D_BB_XLEFT(pl) + POLYGON_2D_BB_WIDTH(pl) / 2.0)
#define POLYGON_2D_BB_YCENTER(pl)   (POLYGON_2D_BB_YTOP(pl) + POLYGON_2D_BB_HEIGHT(pl) / 2.0)	
#define POLYGON_FLAG_MAX(pl)		((pl)->flag_max)

/*
 * 3D polygon: the previous macros are also usable in the following structure
 */

struct polygon_3d {
    int n_points;
    int n_points_max;
    struct point_3d *pts_array;
    struct polygon_3d *next;
};


typedef struct line_2d 		*line_2d_t;     
typedef struct point_2d		*point_2d_t;
typedef struct fpoint_2d	*fpoint_2d_t;
typedef struct point_3d		*point_3d_t;

typedef struct point_2d		Point2D;
typedef struct fpoint_2d        FPoint2D;
typedef struct point_3d		Point3D;
typedef struct vector_2d        Vector2D;
typedef struct fvector_2d       FVector2D;
typedef struct vector_3d        Vector3D;
typedef Point2D			*Point2D_t;
typedef Point3D			*Point3D_t;

typedef struct vector_2d	*vector_2d_t;        
typedef struct fvector_2d       *fvector_2d_t;
typedef struct vector_3d	*vector_3d_t;
typedef struct rectangle	*rect_t;
typedef struct frectangle	*frect_t;
typedef struct h_segment	*h_seg_t;
typedef struct segment_2d	*seg_2d_t;
typedef struct segment_3d	*seg_3d_t;

typedef struct polygon_2d *polygon_2d_t;
typedef struct fpolygon_2d *fpolygon_2d_t;
typedef struct polygon_3d *polygon_3d_t;

typedef struct intersection *intersection_t;

#define LINE_2D_ID(l)		((l)->id)
#define LINE_2D_X1(l)		((l)->x1)
#define LINE_2D_Y1(l)		((l)->y1)
#define LINE_2D_X2(l)		((l)->x2)
#define LINE_2D_Y2(l)		((l)->y2)
#define LINE_2D_NXT(l)		((l)->next)



#define POINT_2D_X(p)		((p)->x)
#define POINT_2D_Y(p)		((p)->y)

#define POINT_3D_X(p)		((p)->x)
#define POINT_3D_Y(p)		((p)->y)
#define POINT_3D_Z(p)		((p)->z)

#define VECTOR_2D_X(vt)		((vt)->x)
#define VECTOR_2D_Y(vt)		((vt)->y)

#define VECTOR_3D_X(vt)		((vt)->x)
#define VECTOR_3D_Y(vt)		((vt)->y)
#define VECTOR_3D_Z(vt)		((vt)->z)

#define H_SEGT_LINE(sg)		((sg)->line)
#define H_SEGT_LEFT(sg)		((sg)->left)
#define H_SEGT_RIGHT(sg)	((sg)->right)
#define H_SEGT_NEXT(sg)		((sg)->next_seg)
#define H_SEGT_LENGTH(sg)	((sg)->right - (sg)->left)

#define SEG_2D_FIRST_POINT(sg)	((sg)->first_point)
#define SEG_2D_LAST_POINT(sg)	((sg)->last_point)
#define SEG_2D_NEXT(sg)		((sg)->next_seg)
#define SEG_2D_FIRST_X(sg)	((sg)->first_point.x)
#define SEG_2D_FIRST_Y(sg)	((sg)->first_point.y)
#define SEG_2D_LAST_X(sg)	((sg)->last_point.x)
#define SEG_2D_LAST_Y(sg)	((sg)->last_point.y)

#define SEG_3D_FIRST_POINT(sg)	((sg)->first_point)
#define SEG_3D_LAST_POINT(sg)	((sg)->last_point)
#define SEG_3D_NEXT(sg)		((sg)->next_seg)
#define SEG_3D_FIRST_X(sg)	((sg)->first_point.x)
#define SEG_3D_FIRST_Y(sg)	((sg)->first_point.y)
#define SEG_3D_FIRST_Z(sg)	((sg)->first_point.z)
#define SEG_3D_LAST_X(sg)	((sg)->last_point.x)
#define SEG_3D_LAST_Y(sg)	((sg)->last_point.y)
#define SEG_3D_LAST_Z(sg)	((sg)->last_point.z)

} // end namespace vsip

#endif /* _GEOMETRIC_H_*/
