/**
 * @file	rectangle.h
 * @brief	rectangle type.
 * @author ORIAN team (orion@sophia.inria.fr)
 */
 
#ifndef _RECTANGLE_H_
#define _RECTANGLE_H_

#include <set>

namespace vsip {

struct rectangle 
{
    int xleft;
    int ytop;
    int xright;
    int ybottom;
    int width;
    int height;
};

struct frectangle 
{
    double xleft;
    double ytop;
    double xright;
    double ybottom;
    double width;
    double height;
};


typedef struct rectangle *rectangle_t;

typedef struct frectangle *frectangle_t;


#define RECT_XLEFT(_rect)	((_rect)->xleft)
#define RECT_X(_rect)		RECT_XLEFT(_rect)
#define RECT_XRIGHT(_rect)	((_rect)->xright)
#define RECT_YTOP(_rect)	((_rect)->ytop)
#define RECT_Y(_rect)		RECT_YTOP(_rect)
#define RECT_YBOTTOM(_rect)	((_rect)->ybottom)
#define RECT_WIDTH(_rect)	((_rect)->width)
#define RECT_HEIGHT(_rect)	((_rect)->height)
#define RECT_XCENTER(_rect)	(RECT_XLEFT(_rect) + RECT_WIDTH(_rect)/2)
#define RECT_YCENTER(_rect)	(RECT_YTOP(_rect) + RECT_HEIGHT(_rect)/2)

rectangle_t 
new_rectangle();


void 
free_rectangle(rectangle_t rect);

rectangle_t 
make_rectangle(int x, 
	       int y, 
	       int w, 
	       int h);

void 
init_rectangle(rectangle_t rect, 
	       int x, 
	       int y, 
	       int w, 
	       int h);

void 
initRectangle(rectangle_t rect, 
	      int x, 
	      int y, 
	      int w, 
	      int h);

rectangle_t 
copy_rectangle(rectangle_t rect);

rectangle_t 
merge_rectangle(rectangle_t rect1, 
		rectangle_t rect2);
		
rectangle_t 
absorb_rectangle(rectangle_t r1, 
				 rectangle_t r2);

int 
rectangle_distance(rectangle_t r1, 
		   rectangle_t r2);

float rectangle_exact_distance(rectangle_t r1, rectangle_t r2);

rectangle_t 
rectangle_intersection(rectangle_t rect1, 
		       rectangle_t rect2);
		       
rectangle_t 
rectangle_union(rectangle_t rect1, 
		rectangle_t rect2);
		
double 
rectangle_intersect_ratio(rectangle_t rec1, 
			  rectangle_t rec2);

double  rectangle_intersect_ratio_strict(rectangle_t rec1, 
								         rectangle_t rec2);

int 
rectangle_in_rectangle(rectangle_t rect1,
		       rectangle_t rect2);

int rectangleIntersectionArea(rectangle_t i_pRectangle1,rectangle_t i_pRectangle2);

int rectangleNIntersectionArea(const std::set<rectangle_t>& i_rectangles);

int rectangleArea(rectangle_t i_pRectangle);


frectangle_t new_frectangle();

void free_frectangle(frectangle_t rect);

frectangle_t make_frectangle(double x, double y, double w, double h);

void init_frectangle(frectangle_t rect, double x, double y, double w, double h);

int rectangles_intersect(rectangle_t r1, rectangle_t r2);

double rect_distance(rectangle_t r1, rectangle_t r2);
					 
} // end namespace vsip
					 
#endif /* _RECTANGLE_H_ */  
