#ifndef SUPABSTRACTDATACALIBRATION_H
#define SUPABSTRACTDATACALIBRATION_H

// SUP
#include "supCoreExport.h"
#include "supAbstractData.h"

// OpenCV
#include <opencv2/core/core.hpp>

const static double height_3d_std = 180.0;

class supAbstractDataCalibrationPrivate;

// /////////////////////////////////////////////////////////////////
// supAbstractDataCalibration interface
// /////////////////////////////////////////////////////////////////

class SUPCORE_EXPORT supAbstractDataCalibration : public supAbstractData
{
    Q_OBJECT

public:
    supAbstractDataCalibration(void);
    supAbstractDataCalibration(const supAbstractDataCalibration& other);
    virtual ~supAbstractDataCalibration(void);

public:
    virtual supAbstractDataCalibration *clone(void);

public:
    supAbstractDataCalibration& operator = (const supAbstractDataCalibration& other);

protected:
    virtual void copy(const dtkAbstractObject& other);

public:
    QString identifier(void) const;

public: // Setters
    void setReferentialName(const QString& i_referentialName);
    void setExtrinsicMatrix(const cv::Mat& i_extrinsic);
    void setDistortionMatrix(const cv::Mat& i_distortion);
    void setIntrinsicMatrix(const cv::Mat& i_intrinsic);

public: // Getters
    const QString& getReferentialName() const;
    const cv::Mat& getExtrinsicMatrix() const;
    const cv::Mat& getDistortionMatrix() const;
    const cv::Mat& getIntrinsicMatrix() const;

public: // Tools
    cv::Mat getCalibration() const;
    cv::Point3d getPosition() const;

    virtual double compute_3d_height(cv::Point2i low_img, double height_2d, double ground[4]) const;
    virtual cv::Point3d imageToWorld(const cv::Point2d i_pos2d,double i_h) const;
    virtual cv::Point2d worldToImage(const cv::Point3d i_pos3d) const;

public: // Read-Write
    virtual bool read(const QString& file);


private:
    DTK_DECLARE_PRIVATE(supAbstractDataCalibration)
};

#endif
