#ifndef SUPVIEWRGBD_P_H
#define SUPVIEWRGBD_P_H

#include <dtkCore/dtkAbstractView_p.h>

class QGraphicsScene;
class MyQGraphicsScene : public QGraphicsScene {

public:

    void keyPressEvent( QKeyEvent* i_event );
    void mousePressEvent( QGraphicsSceneMouseEvent* i_event );

    bool getDisplay () { return this->display; }
    bool getDisplayBlobs () { return this->displayBlobs; }
    bool getDisplayZones () { return this->displayZones; }
    bool getDisplayGroundPlane () { return this->displayGroundPlane; }
    bool getDisplayDepth () { return this->displayDepth; }
    bool getDisplayActors () { return this->displayActors; }
    bool getDisplayTraj () { return this->displayTraj; }
    bool getDisplayEvents () { return this->displayEvents; }
    bool getDisplayActorInfos () { return this->displayActorInfos; }
    bool getDisplayFeet () { return this->displayFeet; }
    bool getDisplayFrameRate () { return this->displayFrameRate; }
    bool getDisplayListOfPointsDEBUG () { return this->displayListOfPointsDEBUG; }
    bool getDisplayGroundPoints () { return this->displayGroundPoints; }


    int getTrajSize () { return this->TRAJ_SIZE; }

    void setDisplayDepth () { this->displayDepth = !this->displayDepth; }

private:

    bool display = true; // Display the image
    bool displayBlobs = true; // Display blobs or not
    bool displayZones = false; // Display zones or not
    bool displayGroundPlane = false; // Display ground plane or not
    bool displayDepth = false; // Display depth or color image
    bool displayActors = false; // Display actors
    bool displayTraj = true; // Display trajectory
    bool displayEvents = true; // Display event list
    bool displayActorInfos = false; // Display actor informations
    bool displayFeet = true; // Display actor feet
    bool displayFrameRate = true; // Display the frame rate
    bool displayListOfPointsDEBUG = false; // Display a list of 3D points
    bool displayGroundPoints = false; // Display the list of ground points

    int TRAJ_SIZE = 30; // Size of the trajectory to display

};

class QGraphicsView;

class QGraphicsPixmapItem;
class QGraphicsItemGroup;

class supViewRGBD;

class supViewRGBDPrivate : public dtkAbstractViewPrivate
{
public:
    supViewRGBDPrivate(supViewRGBD *q = 0) : dtkAbstractViewPrivate(q), image(new QGraphicsPixmapItem), depth(new QGraphicsPixmapItem), segmentation( new QGraphicsPixmapItem)
    {
        view.setRenderHint(QPainter::Antialiasing, true);
        view.setScene(&scene);

        scene.addItem(image);
        scene.addItem(segmentation);
    }
    supViewRGBDPrivate(const supViewRGBDPrivate& other) : dtkAbstractViewPrivate(other), image(other.image), depth(other.depth), segmentation(other.segmentation), display(other.display)
    {
        view.setRenderHint(QPainter::Antialiasing, true);
        view.setScene(&scene);

        scene.addItem(image);
        scene.addItem(segmentation);
    }

public:
    virtual ~supViewRGBDPrivate(void)
    {
        scene.clear();
    }

public:
    MyQGraphicsScene scene;
    QGraphicsView view;

    QSharedPointer <supDataImage> acquisitionImage;
    QSharedPointer <supDataImage> acquisitionDepth;
    supAbstractDataCalibrationWithDepth calibrationWithDepth;
    std::list <ContextElement> contextElems;
    std::vector<double> groundPlaneCoef;
    std::list <QSharedPointer < const supDataActor > > actors;
    std::list <QSharedPointer < supDataEvent > > events;
    std::map <int, std::pair <cv::Point3f, cv::Point3f> > listOfFeetPositions;
    std::list <cv::Point3f> listOfPointsDEBUG;
    std::list <cv::Point3f> listOfGroundPoints;

    std::vector<QColor> colors;

    QGraphicsPixmapItem* image;
    QGraphicsPixmapItem* depth;
    QGraphicsPixmapItem* segmentation;
    QList< QGraphicsItem* > blobs;
    QList< QGraphicsItem* > pos;

    QSharedPointer< supDataImage > display;

    int frameRate;

};

DTK_IMPLEMENT_PRIVATE(supViewRGBD, dtkAbstractView)

#endif

